@@ -1,4 +1,4 @@
-/* $Header: /local/src/CVS/nickle/builtin-toplevel.c,v 1.23 2004/04/16 05:26:44 keithp Exp $ */
+/* $Header: /local/src/CVS/nickle/builtin-toplevel.c,v 1.25 2004/05/13 21:16:52 bart Exp $ */
 
 /*
  * Copyright © 1988-2004 Keith Packard and Bart Massey.
@@ -53,6 +53,7 @@ import_Toplevel_namespace()
         { do_is_thread, "is_thread", "b", "p" },
         { do_is_bool, "is_bool", "b", "p" },
         { do_is_void, "is_void", "b", "p" },
+        { do_is_uninit, "is_uninit", "b", "*p" },
         { do_mantissa, "mantissa", "r", "R" },
         { do_numerator, "numerator", "i", "R" },
         { do_precision, "precision", "i", "R" },
@@ -700,6 +701,23 @@ do_is_void (Value av)
     RETURN (av);
 }
 
+Value
+do_is_uninit (Value av)
+{
+    ENTER ();
+    if (!av) {
+	RaiseStandardException (exception_invalid_argument,
+				"do_is_uninit: invalid reference",
+				2, NewInt (0), av);
+	av = Void;
+    } else if (RefValueGet(av)) {
+	av = FalseVal;
+    } else {
+	av = TrueVal;
+    }
+    RETURN (av);
+}
+
 Value
 do_is_array (Value av)
 {
