@@ -1,4 +1,4 @@
-/* $Header: /local/src/CVS/nickle/pretty.c,v 1.66 2004/04/11 06:06:52 keithp Exp $ */
+/* $Header: /local/src/CVS/nickle/pretty.c,v 1.69 2004/05/20 09:51:55 bart Exp $ */
 
 /*
  * Copyright © 1988-2004 Keith Packard and Bart Massey.
@@ -656,22 +656,25 @@ PrettyStatement (Value f, Expr *e, int level, int blevel, Bool nest, ProfileData
     case FOR:
 	PrettyIndent (f, e, level, pd);
 	FilePuts (f, "for (");
-	PrettyExpr (f, e->tree.left->tree.left, -1, level, nest, pd);
-	FilePuts (f, ";");
-	if (e->tree.left->tree.right)
+	if (e->tree.left->tree.left)
+	    PrettyExpr (f, e->tree.left->tree.left, -1, level, nest, pd);
+	if (e->tree.left->base.tag == SEMI)
+	    FilePuts (f, ";");
+	if (e->tree.left->tree.right->tree.left)
 	{
-	    FilePuts (f, " ");
-	    PrettyExpr (f, e->tree.left->tree.right, -1, level, nest, pd);
+	    if (e->tree.left->base.tag == SEMI)
+		FilePuts (f, " ");
+	    PrettyExpr (f, e->tree.left->tree.right->tree.left, -1, level, nest, pd);
 	}
 	FilePuts (f, ";");
-	if (e->tree.right->tree.left)
+	if (e->tree.left->tree.right->tree.right->tree.left)
 	{
 	    FilePuts (f, " ");
-	    PrettyExpr (f, e->tree.right->tree.left, -1, level, nest, pd);
+	    PrettyExpr (f, e->tree.left->tree.right->tree.right->tree.left, -1, level, nest, pd);
 	}
 	FilePuts (f, ")\n");
 	if (nest)
-	    PrettyStatement (f, e->tree.right->tree.right, level+1, level, nest, pd);
+	    PrettyStatement (f, e->tree.right, level+1, level, nest, pd);
 	break;
     case SWITCH:
     case UNION:
@@ -894,7 +897,7 @@ doPrettyPrint (Value f, Publish publish, SymbolPtr symbol, int level, Bool nest)
     if (!symbol)
 	return;
     if (profiling)
-	FilePuts (f, "    called       self\n");
+	FilePuts (f, "    called(ms)   self(ms)\n");
     PrettyIndent (f, 0, level, 0);
     switch (symbol->symbol.class) {
     case class_const:
