@@ -1,4 +1,4 @@
-/* $Header: /local/src/CVS/nickle/scope.c,v 1.29 2004/04/16 05:26:45 keithp Exp $ */
+/* $Header: /local/src/CVS/nickle/scope.c,v 1.30 2004/04/16 21:41:52 keithp Exp $ */
 
 /*
  * Copyright © 1988-2004 Keith Packard and Bart Massey.
@@ -261,8 +261,8 @@ NamespaceLocate (Value		names,
 	if (!namelist)
 	{
 	    if (complain)
-		FilePrintf (FileStdout, "No symbol \"%s\" in namespace\n",
-			    StringChars (&string->string));
+		FilePrintf (FileStdout, "No symbol %v in namespace\n",
+			    string);
 	    return False;
 	}
 	symbol = namelist->symbol;
